import org.netlib.blas.*;

public class KktDistanceMatrix {

	public static void main(String[] args) {
		test();
	}

	public static void test(){

		double[][] v1 = {{2d, 1d}};
		double[][] v2 = {{1d,1d},{1d,0d}};
	
		long time_before = System.currentTimeMillis(); 
		for (int i = 0  ;  i < 10000  ; ++i ) {
			double[][] dm = kktDistanceMatrix(v1, v2);
		}
		long time_after = System.currentTimeMillis();
		System.out.println("Elapsed : " + (double)(time_after-time_before)/1000 + "s.");
	}


	public static double dot(double[] x, double[] y){
		return DDOT.DDOT(Math.min(x.length,y.length), x, 1, y, 1);
	}
	

	public static double[][] kktDistanceMatrix(double[][] simplex1, double[][] simplex2){
	  
		final int n = simplex1.length;
		final int m = simplex2.length;
		final int nm = n+m;
		final int nm1 = 1+nm;
		final int nm2 = 1+nm1;
		final double[][] result = new double[nm2][nm2];
		double mij = 0d;

		for ( int i = 0  ;  i<n  ; ++i ){
			result[i][nm] = 1d;
			result[nm][i] = 1d;
			{
				double[] ai =  simplex1[i];
	
				mij = dot(ai , ai);
				result[i][i] = mij;

				for ( int j = i+1  ;  j < n  ;  ++j ){
					mij = dot(ai, simplex1[j]);
					result[i][j] = mij;
					result[j][i] = mij;
				}
					
				for ( int nj = n, j=0  ;  nj < nm  ;  ++nj, ++j )
				{
					mij = - dot(ai, simplex2[j]);
					result[i][nj] = mij;
					result[nj][i] = mij;
				}
			}
		}
		
		for (int nj = n, j = 0  ;  nj < nm  ;  ++nj,++j )
		{
			result[nj][nm1] = 1d;
			result[nm1][nj] = 1d;
			{
				double[] bj = simplex2[j];
				mij = dot(bj , bj);
				for (int njcol = n+1, jcol=1  ;  njcol < nm  ;  ++njcol, ++jcol)
				{
					mij = dot(bj, simplex2[jcol]);
					result[nj][njcol] = mij;
					result[njcol][nj] = mij;
				}
			}
		}
		return result;
	}
}
//EOF
