(require :blas)

(defun test ()
  (let ((v1 (make-array 1 :element-type '(array double-float 1) 
  			:initial-contents (list
				(make-array 2 :element-type 'double-float 
					:initial-contents '(2d0 1d0) ))))
       (v2 (make-array 2 :element-type '(array double-float 1) 
			:initial-contents (list

				(make-array 2 :element-type 'double-float 
					:initial-contents '(1d0 1d0))

				(make-array 2 :element-type 'double-float 
					:initial-contents '(1d0 0d0)) ))))
   (time (dotimes (i 10000) 
	   (kkt-distance-matrix v1 v2))))
)


(defun dot (x y)

  (declare (type  (simple-array * 1) x y))
  (declare (ftype (function ((simple-array * 1) (simple-array * 1)) double-float) 
		dot))

  (let* ((n (min (length x) (length y)))
	 (result (blas:ddot n x 1 y 1)))

    (declare (type fortran-to-lisp::integer4 n)
	     (type double-float result))
    
    result )
)


(defun kkt-distance-matrix (simplex1 simplex2)
" Computes the KKT-matrix for the distance function between 
  simplex1 and simplex2 where no vertex is active.
"
(declare (type (simple-array (simple-array * 1) 1) simplex1 simplex2))
(declare (ftype (function ((simple-array (simple-array * 1) 1) 
			   (simple-array (simple-array * 1) 1))
			  (simple-array double-float 2))
		kkt-distance-matrix))
(declare (optimize (speed 3)))

  (let* ((n   (length simplex1))
	 (m   (length simplex2))
	 (n+m   (+ n m))
	 (n+m+1 (1+ n+m))
	 (n+m+2 (1+ n+m+1))
	 (result (make-array `(,n+m+2 ,n+m+2) 
			     :element-type 'double-float
			     :initial-element 0d0))
	 (mij 0d0))

    (declare (type fixnum n m n+m n+m+1 n+m+2)
	     (type double-float mij)
	     (type (simple-array double-float 2) result))

    (dotimes (i n) (declare (type fixnum i))
      (setf (aref result i   n+m) 1d0
	    (aref result n+m i)   1d0)
      (let 
	  ((ai (aref simplex1 i)))
	(declare (type (simple-array * 1) ai))
	(setf mij                (dot ai ai)
	      (aref result i i)  mij)
	(loop :for j :from (1+ i) :below n
	   :do
	   (setf mij               (dot ai (aref simplex1 j))
		 (aref result i j) mij
		 (aref result j i) mij))
	(loop :for n+j :from n :below n+m
	   :for j = 0 :then (1+ j)
	   :do
	   (setf mij            (- (dot ai (aref simplex2 j)))
		 (aref result i   n+j) mij
		 (aref result n+j i)   mij))))
    (loop :for n+j fixnum :from n :below n+m
       :for j fixnum = 0 :then (1+ j)
       :do
       (setf (aref result n+j   n+m+1) 1d0
	     (aref result n+m+1 n+j)   1d0)     
       (let ((bj (aref simplex2 j)))
	 (setf (aref result n+j n+j) (dot bj bj))
	 (loop :for n+j-col :from (1+ n+j) :below n+m
	    :for j-col = 1 :then (1+ j-col)
	    :do
	    (setf mij              (dot bj (aref simplex2 j-col))
		  (aref result n+j      n+j-col) mij
		  (aref result n+j-col n+j)      mij))))
    result ))

;;; EOF
